extends SSTVEncoder
class_name Martin1

const COLOR_FREQ_MULT = 3.1372549

func PixelRGBAsFrequencies(image: Image, scanLine: int, scanLinePos: int) -> Array:
	var color = image.get_pixel(scanLinePos, scanLine)
	var redFreq = float(color.r8) * COLOR_FREQ_MULT + 1500.0
	var greenFreq = float(color.g8) * COLOR_FREQ_MULT + 1500.0
	var blueFreq = float(color.b8) * COLOR_FREQ_MULT + 1500.0
	return [redFreq, greenFreq, blueFreq]

const IMAGE_WIDTH = 320.0
const IMAGE_HEIGHT = 256.0
const BLANKING_INTERVAL = 0.000572
const SCAN_LINE_LENGTH = 0.146432
const SYNC_PULSE_LENGTH = 0.004862

var preparedImage = []

func PrepareImage(image: Image):
	image.resize(IMAGE_WIDTH, IMAGE_HEIGHT)
	image.convert(Image.FORMAT_RGB8)

	for scanLine in range(0, IMAGE_HEIGHT):
		var red = []
		var green = []
		var blue = []
		for vertPos in range(0, IMAGE_WIDTH):
			var freqs = PixelRGBAsFrequencies(image, scanLine, vertPos)
			red.push_back(freqs[0])
			green.push_back(freqs[1])
			blue.push_back(freqs[2])
		preparedImage.push_back([green, blue, red])

func EncodeSSTV(image: Image, skipHeader: bool = false):
	PrepareImage(image)
	
	if not skipHeader:
		EncodePrefix()
		EncodeHeader([false, true, false, true, true, false, false])

	for scanLine in range(0, IMAGE_HEIGHT):
		outputFloats.append_array(GenerateTone(SYNC_PULSE_FREQ, SYNC_PULSE_LENGTH))
		outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, BLANKING_INTERVAL))
		for dataLine in range(0, 3):
			outputFloats.append_array(GenerateToneFromCurve(preparedImage[scanLine][dataLine], SCAN_LINE_LENGTH))
			outputFloats.append_array(GenerateTone(BLANKING_PULSE_FREQ, BLANKING_INTERVAL))
	
	return outputFloats
